@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, Intel Corporation
+ * Copyright (c) 2017 - 2018, Intel Corporation
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,6 +27,7 @@
 #include <errno.h>
 #include <inttypes.h>
 #include <stdbool.h>
+#include <string.h>
 
 #include "tabrmd.h"
 
@@ -36,6 +37,24 @@
 #include "tpm2-response.h"
 #include "util.h"
 
+#define TPMS_CAPABILITY_DATA_ZERO_INIT { 0, { 0, { \
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 } } }
+
 G_DEFINE_TYPE (AccessBroker, access_broker, G_TYPE_OBJECT);
 
 enum {
@@ -101,7 +120,7 @@ access_broker_get_property (GObject     *object,
     }
 }
 /*
- * Dispose function: relese references to gobjects. We also free the SAPI
+ * Dispose function: release references to gobjects. We also free the SAPI
  * context here as well. Typically freeing this data would be done in the
  * finalize function but it can't be used w/o the underlying TCTI.
  */
@@ -122,7 +141,10 @@ access_broker_dispose (GObject *obj)
  */
 static void
 access_broker_init (AccessBroker *broker)
-{ /* noop */ }
+{
+    UNUSED_PARAM(broker);
+    /* noop */
+}
 /**
  * GObject class initialization function. This function boils down to:
  * - Setting up the parent class.
@@ -155,6 +177,15 @@ access_broker_class_init (AccessBrokerClass *klass)
                                        N_PROPERTIES,
                                        obj_properties);
 }
+
+#define SUPPORTED_ABI_VERSION \
+{ \
+    .tssCreator = 1, \
+    .tssFamily = 2, \
+    .tssLevel = 1, \
+    .tssVersion = 108, \
+}
+
 static TSS2_SYS_CONTEXT*
 sapi_context_init (Tcti *tcti)
 {
@@ -162,12 +193,7 @@ sapi_context_init (Tcti *tcti)
     TSS2_TCTI_CONTEXT *tcti_context;
     TSS2_RC rc;
     size_t size;
-    TSS2_ABI_VERSION abi_version = {
-        .tssCreator = TSSWG_INTEROP,
-        .tssFamily  = TSS_SAPI_FIRST_FAMILY,
-        .tssLevel   = TSS_SAPI_FIRST_LEVEL,
-        .tssVersion = TSS_SAPI_FIRST_VERSION,
-    };
+    TSS2_ABI_VERSION abi_version = SUPPORTED_ABI_VERSION;
 
     g_debug ("sapi_context_init w/ Tcti: 0x%" PRIxPTR, (uintptr_t)tcti);
     tcti_context = tcti_peek_context (tcti);
@@ -194,8 +220,8 @@ access_broker_send_tpm_startup (AccessBroker *broker)
 {
     TSS2_RC rc;
 
-    rc = Tss2_Sys_Startup (broker->sapi_context, TPM_SU_CLEAR);
-    if (rc != TSS2_RC_SUCCESS && rc != TPM_RC_INITIALIZE)
+    rc = Tss2_Sys_Startup (broker->sapi_context, TPM2_SU_CLEAR);
+    if (rc != TSS2_RC_SUCCESS && rc != TPM2_RC_INITIALIZE)
         g_warning ("Tss2_Sys_Startup returned unexpected RC: 0x%" PRIx32, rc);
     else
         rc = TSS2_RC_SUCCESS;
@@ -214,6 +240,11 @@ access_broker_lock (AccessBroker *broker)
 {
     gint error;
 
+    if (broker == NULL) {
+        g_error ("AccessBroker: NULL pointer passed to access_broker_lock");
+        return;
+    }
+
     error = pthread_mutex_lock (&broker->sapi_mutex);
     if (error != 0) {
         switch (error) {
@@ -221,7 +252,7 @@ access_broker_lock (AccessBroker *broker)
             g_error ("AccessBroker: attempted to lock uninitialized mutex");
             break;
         default:
-            g_error ("AccessBroker: unkonwn error attempting to lock SAPI "
+            g_error ("AccessBroker: unknown error attempting to lock SAPI "
                      "mutex: 0x%x", error);
             break;
         }
@@ -239,6 +270,11 @@ access_broker_unlock (AccessBroker *broker)
 {
     gint error;
 
+    if (broker == NULL) {
+        g_error ("AccessBroker: NULL pointer passed to access_broker_unlock");
+        return;
+    }
+
     error= pthread_mutex_unlock (&broker->sapi_mutex);
     if (error != 0) {
         switch (error) {
@@ -253,7 +289,7 @@ access_broker_unlock (AccessBroker *broker)
     }
 }
 /**
- * Query the TPM for fixed (PT_FIXED) TPM properties.
+ * Query the TPM for fixed (TPM2_PT_FIXED) TPM properties.
  * This function is intended for internal use only. The caller MUST
  * hold the sapi_mutex lock before calling.
  */
@@ -271,21 +307,21 @@ access_broker_get_tpm_properties_fixed (TSS2_SYS_CONTEXT     *sapi_context,
         return TSS2_RESMGR_RC_SAPI_INIT;
     rc = Tss2_Sys_GetCapability (sapi_context,
                                  NULL,
-                                 TPM_CAP_TPM_PROPERTIES,
-                                 /* get PT_FIXED TPM property group */
-                                 PT_FIXED,
+                                 TPM2_CAP_TPM_PROPERTIES,
+                                 /* get TPM2_PT_FIXED TPM property group */
+                                 TPM2_PT_FIXED,
                                  /* get all properties in the group */
-                                 MAX_TPM_PROPERTIES,
+                                 TPM2_MAX_TPM_PROPERTIES,
                                  &more_data,
                                  capability_data,
                                  NULL);
     if (rc != TSS2_RC_SUCCESS) {
-        g_error ("Failed to GetCapability: TPM_CAP_TPM_PROPERTIES, "
-                 "PT_FIXED: 0x%x", rc);
+        g_error ("Failed to GetCapability: TPM2_CAP_TPM_PROPERTIES, "
+                 "TPM2_PT_FIXED: 0x%x", rc);
         return rc;
     }
     /* sanity check the property returned */
-    if (capability_data->capability != TPM_CAP_TPM_PROPERTIES)
+    if (capability_data->capability != TPM2_CAP_TPM_PROPERTIES)
         g_warning ("GetCapability returned wrong capability: 0x%x",
                    capability_data->capability);
     return rc;
@@ -298,10 +334,10 @@ access_broker_get_tpm_properties_fixed (TSS2_SYS_CONTEXT     *sapi_context,
  */
 TSS2_RC
 access_broker_get_fixed_property (AccessBroker           *broker,
-                                  TPM_PT                  property,
+                                  TPM2_PT                  property,
                                   guint32                *value)
 {
-    gint i;
+    unsigned int i;
 
     if (broker->properties_fixed.data.tpmProperties.count == 0) {
         return TSS2_RESMGR_RC_INTERNAL_ERROR;
@@ -324,41 +360,47 @@ access_broker_get_fixed_property (AccessBroker           *broker,
 TSS2_SYS_CONTEXT*
 access_broker_lock_sapi (AccessBroker *broker)
 {
+    if (broker == NULL) {
+        g_error (
+            "AccessBroker: NULL pointer passed to access_broker_lock_sapi");
+        return NULL;
+    }
+
     access_broker_lock (broker);
     g_assert_nonnull (broker->sapi_context);
     return broker->sapi_context;
 }
 /*
- * Return the TPM_PT_TOTAL_COMAMNDS fixed TPM property.
+ * Return the TPM2_PT_TOTAL_COMAMNDS fixed TPM property.
  */
 TSS2_RC
 access_broker_get_total_commands (AccessBroker *broker,
                                   guint        *value)
 {
     return access_broker_get_fixed_property (broker,
-                                             TPM_PT_TOTAL_COMMANDS,
+                                             TPM2_PT_TOTAL_COMMANDS,
                                              value);
 }
 /**
- * Return the TPM_PT_MAX_COMMAND_SIZE fixed TPM property.
+ * Return the TPM2_PT_TPM2_MAX_COMMAND_SIZE fixed TPM property.
  */
 TSS2_RC
 access_broker_get_max_command (AccessBroker *broker,
                                guint32      *value)
 {
     return access_broker_get_fixed_property (broker,
-                                             TPM_PT_MAX_COMMAND_SIZE,
+                                             TPM2_PT_MAX_COMMAND_SIZE,
                                              value);
 }
 /**
- * Return the TPM_PT_MAX_RESPONSE_SIZE fixed TPM property.
+ * Return the TPM2_PT_TPM2_MAX_RESPONSE_SIZE fixed TPM property.
  */
 TSS2_RC
 access_broker_get_max_response (AccessBroker *broker,
                                 guint32      *value)
 {
     return access_broker_get_fixed_property (broker,
-                                             TPM_PT_MAX_RESPONSE_SIZE,
+                                             TPM2_PT_MAX_RESPONSE_SIZE,
                                              value);
 }
 /* Send the parameter Tpm2Command to the TPM. Return the TSS2_RC. */
@@ -402,7 +444,7 @@ access_broker_get_response (AccessBroker *broker,
     if (*buffer == NULL) {
         g_warning ("failed to allocate buffer for Tpm2Response: %s",
                    strerror (errno));
-        return RM_RC (TPM_RC_MEMORY);
+        return RM_RC (TPM2_RC_MEMORY);
     }
     *buffer_size = max_size;
     rc = tcti_receive (broker->tcti, buffer_size, *buffer, TSS2_TCTI_TIMEOUT_BLOCK);
@@ -413,7 +455,7 @@ access_broker_get_response (AccessBroker *broker,
 /**
  * In the most simple case the caller will want to send just a single
  * command represented by a Tpm2Command object. The response is passed
- * back as the return value. The resonse code is returend through the
+ * back as the return value. The response code is returned through the
  * 'rc' out parameter.
  * The caller MUST NOT hold the lock when calling. This function will take
  * the lock for itself.
@@ -454,7 +496,7 @@ access_broker_send_command (AccessBroker  *broker,
     g_debug ("access_broker_send_command: AccessBroker: 0x%" PRIxPTR
              " Tpm2Response: 0x%" PRIxPTR " RC: 0x%" PRIx32, (uintptr_t)broker,
              (uintptr_t)response, tpm2_response_get_code (response));
-    g_object_unref (connection);
+    g_clear_object (&connection);
     return response;
 
 unlock_out:
@@ -490,7 +532,7 @@ access_broker_new (Tcti *tcti)
  * Initialize the AccessBroker. This is all about initializing internal data
  * that normally we would want to do in a constructor. But since this
  * initialization requires reaching out to the TPM and could fail we don't
- * want to do it in the constructur / _new function. So we put it here in
+ * want to do it in the constructor / _new function. So we put it here in
  * an explicit _init function that must be executed after the object has
  * been instantiated.
  */
@@ -505,8 +547,7 @@ access_broker_init_tpm (AccessBroker *broker)
     pthread_mutex_init (&broker->sapi_mutex, NULL);
     rc = access_broker_send_tpm_startup (broker);
     if (rc != TSS2_RC_SUCCESS) {
-        g_error ("access_broker_sent_tpm_startup failed: 0x%x", rc);
-        goto out;
+        g_warning ("access_broker_sent_tpm_startup failed: 0x%x", rc);
     }
     rc = access_broker_get_tpm_properties_fixed (broker->sapi_context,
                                                  &broker->properties_fixed);
@@ -528,27 +569,28 @@ access_broker_get_trans_object_count (AccessBroker *broker,
     TSS2_RC rc = TSS2_RC_SUCCESS;
     TSS2_SYS_CONTEXT *sapi_context;
     TPMI_YES_NO more_data;
-    TPMS_CAPABILITY_DATA capability_data = { 0, };
+    TPMS_CAPABILITY_DATA capability_data = TPMS_CAPABILITY_DATA_ZERO_INIT;
 
     g_assert_nonnull (broker);
     g_assert_nonnull (count);
     sapi_context = access_broker_lock_sapi (broker);
     /*
-     * GCC gets confused by the TRANSIENT_FIRST constant being used for the
-     * 4th parameter. It assumes that it's a signed type which causes
+     * GCC gets confused by the TPM2_TRANSIENT_FIRST constant being used for
+     * the 4th parameter. It assumes that it's a signed type which causes
      * -Wsign-conversion to complain. Casting to UINT32 is all we can do.
      */
     rc = Tss2_Sys_GetCapability (sapi_context,
                                  NULL,
-                                 TPM_CAP_HANDLES,
-                                 (UINT32)TRANSIENT_FIRST,
-                                 TRANSIENT_LAST - TRANSIENT_FIRST,
+                                 TPM2_CAP_HANDLES,
+                                 (UINT32)TPM2_TRANSIENT_FIRST,
+                                 TPM2_TRANSIENT_LAST - TPM2_TRANSIENT_FIRST,
                                  &more_data,
                                  &capability_data,
                                  NULL);
     if (rc != TSS2_RC_SUCCESS)
         goto out;
-    *count = capability_data.data.handles.count;
+    if (count != NULL)
+        *count = capability_data.data.handles.count;
 out:
     access_broker_unlock (broker);
     return rc;
@@ -556,7 +598,7 @@ out:
 TSS2_RC
 access_broker_context_load (AccessBroker *broker,
                             TPMS_CONTEXT *context,
-                            TPM_HANDLE   *handle)
+                            TPM2_HANDLE   *handle)
 {
     TSS2_RC           rc;
     TSS2_SYS_CONTEXT *sapi_context;
@@ -587,7 +629,7 @@ access_broker_context_load (AccessBroker *broker,
  */
 TSS2_RC
 access_broker_context_save (AccessBroker *broker,
-                            TPM_HANDLE    handle,
+                            TPM2_HANDLE    handle,
                             TPMS_CONTEXT *context)
 {
     TSS2_RC rc;
@@ -599,6 +641,9 @@ access_broker_context_save (AccessBroker *broker,
     g_debug ("access_broker_context_save: handle 0x%08" PRIx32, handle);
     sapi_context = access_broker_lock_sapi (broker);
     rc = Tss2_Sys_ContextSave (sapi_context, handle, context);
+    if (rc != TSS2_RC_SUCCESS) {
+        g_warning ("%s returned an error: 0x%" PRIx32, __func__, rc);
+    }
     access_broker_unlock (broker);
 
     return rc;
@@ -608,7 +653,7 @@ access_broker_context_save (AccessBroker *broker,
  */
 TSS2_RC
 access_broker_context_flush (AccessBroker *broker,
-                             TPM_HANDLE    handle)
+                             TPM2_HANDLE    handle)
 {
     TSS2_RC rc;
     TSS2_SYS_CONTEXT *sapi_context;
@@ -629,7 +674,7 @@ access_broker_context_flush (AccessBroker *broker,
 }
 TSS2_RC
 access_broker_context_saveflush (AccessBroker *broker,
-                                 TPM_HANDLE    handle,
+                                 TPM2_HANDLE    handle,
                                  TPMS_CONTEXT *context)
 {
     TSS2_RC           rc;
@@ -642,15 +687,17 @@ access_broker_context_saveflush (AccessBroker *broker,
     sapi_context = access_broker_lock_sapi (broker);
     rc = Tss2_Sys_ContextSave (sapi_context, handle, context);
     if (rc != TSS2_RC_SUCCESS) {
-        g_warning ("Tss2_Sys_ContextSave: failed to save context for "
-                   "handle: 0x%" PRIx32 " TSS2_RC: 0x%" PRIx32, handle, rc);
+        g_warning ("%s: Tss2_Sys_ContextSave failed to save context for "
+                   "handle: 0x%" PRIx32 " TSS2_RC: 0x%" PRIx32, __func__,
+                   handle, rc);
         goto out;
     }
     g_debug ("access_broker_context_saveflush: handle 0x%" PRIx32, handle);
     rc = Tss2_Sys_FlushContext (sapi_context, handle);
-    if (rc != TSS2_RC_SUCCESS)
-        g_warning("Tss2_Sys_FlushContext: failed to flushed context for "
-                  "handle: 0x%" PRIx32 ", TSS2_RC: 0x%" PRIx32, handle, rc);
+    if (rc != TSS2_RC_SUCCESS) {
+        g_warning("%s: Tss2_Sys_FlushContext failed for handle: 0x%" PRIx32
+                  ", TSS2_RC: 0x%" PRIx32, __func__, handle, rc);
+    }
 out:
     access_broker_unlock (broker);
     return rc;
@@ -658,27 +705,27 @@ out:
 static void
 access_broker_flush_all_unlocked (AccessBroker     *broker,
                                   TSS2_SYS_CONTEXT *sapi_context,
-                                  TPM_RH            first,
-                                  TPM_RH            last)
+                                  TPM2_RH            first,
+                                  TPM2_RH            last)
 {
     TSS2_RC rc = TSS2_RC_SUCCESS;
     TPMI_YES_NO more_data;
-    TPMS_CAPABILITY_DATA capability_data = { 0, };
-    TPM_HANDLE handle;
+    TPMS_CAPABILITY_DATA capability_data = TPMS_CAPABILITY_DATA_ZERO_INIT;
+    TPM2_HANDLE handle;
     size_t i;
 
     g_debug ("%s: AccessBroker: 0x%" PRIxPTR ", first: 0x%08" PRIx32 ", last: "
              "0x%08" PRIx32, __func__, (uintptr_t)broker, first, last);
     rc = Tss2_Sys_GetCapability (sapi_context,
                                  NULL,
-                                 TPM_CAP_HANDLES,
+                                 TPM2_CAP_HANDLES,
                                  first,
                                  last - first,
                                  &more_data,
                                  &capability_data,
                                  NULL);
     if (rc != TSS2_RC_SUCCESS) {
-        g_warning ("Failed to get capability TPM_CAP_HANDLES");
+        g_warning ("Failed to get capability TPM2_CAP_HANDLES");
         return;
     }
     g_debug ("%s: got %u handles", __func__, capability_data.data.handles.count);
@@ -703,15 +750,15 @@ access_broker_flush_all_context (AccessBroker *broker)
     sapi_context = access_broker_lock_sapi (broker);
     access_broker_flush_all_unlocked (broker,
                                       sapi_context,
-                                      ACTIVE_SESSION_FIRST,
-                                      ACTIVE_SESSION_LAST);
+                                      TPM2_ACTIVE_SESSION_FIRST,
+                                      TPM2_ACTIVE_SESSION_LAST);
     access_broker_flush_all_unlocked (broker,
                                       sapi_context,
-                                      LOADED_SESSION_FIRST,
-                                      LOADED_SESSION_LAST);
+                                      TPM2_LOADED_SESSION_FIRST,
+                                      TPM2_LOADED_SESSION_LAST);
     access_broker_flush_all_unlocked (broker,
                                       sapi_context,
-                                      TRANSIENT_FIRST,
-                                      TRANSIENT_LAST);
+                                      TPM2_TRANSIENT_FIRST,
+                                      TPM2_TRANSIENT_LAST);
     access_broker_unlock (broker);
 }
