@@ -28,12 +28,14 @@
 #include <string.h>
 #include <inttypes.h>
 
+#include "util.h"
 #include "session-list.h"
 
 G_DEFINE_TYPE (SessionList, session_list, G_TYPE_OBJECT);
 
 enum {
     PROP_0,
+    PROP_MAX_ABANDONED,
     PROP_MAX_PER_CONNECTION,
     N_PROPERTIES
 };
@@ -50,6 +52,9 @@ session_list_get_property (GObject    *object,
     SessionList *list = SESSION_LIST (object);
 
     switch (property_id) {
+    case PROP_MAX_ABANDONED:
+        g_value_set_uint (value, list->max_abandoned);
+        break;
     case PROP_MAX_PER_CONNECTION:
         g_value_set_uint (value, list->max_per_connection);
         break;
@@ -70,7 +75,12 @@ session_list_set_property (GObject        *object,
     SessionList *list = SESSION_LIST (object);
 
     switch (property_id) {
-    case PROP_MAX_PER_CONNECTION:
+    case PROP_MAX_ABANDONED:
+        list->max_abandoned = g_value_get_uint (value);
+        g_debug ("%s: 0x%" PRIxPTR " max-abandoned: %u",
+                 __func__, (uintptr_t)list, list->max_abandoned);
+        break;
+   case PROP_MAX_PER_CONNECTION:
         list->max_per_connection = g_value_get_uint (value);
         g_debug ("session_list_set_property: 0x%" PRIxPTR
                  " max-per-connection: %u", (intptr_t)list,
@@ -82,19 +92,15 @@ session_list_set_property (GObject        *object,
     }
 }
 /*
- * Initialize object. This requires:
- * 1) initializing the mutex that mediate access to the hash tables
- * 2) creating the hash tables
- * 3) initializing the handle_count
- * The handle_count is currently initialized to start allocating handles
- * @ 0xff. This is an arbitrary way we differentiate them from the handles
- * allocated by the TPM.
+ * Initialize object.
+ * GQueue for 'abandoned_queue' must be explicitly created.
+ * GList for 'session_entry_list' does not.
  */
 static void
 session_list_init (SessionList     *list)
 {
     g_debug ("session_list_init");
-    pthread_mutex_init (&list->mutex, NULL);
+    list->abandoned_queue = g_queue_new ();
     list->session_entry_list = NULL;
 }
 /*
@@ -110,16 +116,14 @@ session_list_dispose (GObject *object)
     g_debug ("%s: SessionList: 0x%" PRIxPTR " with %" PRIu32 " entries",
              __func__, (uintptr_t)self,
              g_list_length (self->session_entry_list));
+    g_queue_free (self->abandoned_queue);
+    self->abandoned_queue = NULL;
     g_list_free_full (self->session_entry_list, g_object_unref);
     self->session_entry_list = NULL;
     G_OBJECT_CLASS (session_list_parent_class)->dispose (object);
 }
 /*
- * Deallocate all associated resources. The mutex may be held by another
- * thread. If we attempt to lock / unlock it we may hang a thread in the
- * guts of the GObject system. Better to just destroy it and free all
- * other resources. The call to destroy may fail and if so we just carry
- * on.
+ * Deallocate all associated resources.
  */
 static void
 session_list_finalize (GObject *object)
@@ -129,7 +133,6 @@ session_list_finalize (GObject *object)
     g_debug ("session_list_finalize: SessionList: 0x%" PRIxPTR " with %"
              PRIu32 " entries", (uintptr_t)self,
              g_list_length (self->session_entry_list));
-    pthread_mutex_destroy (&self->mutex);
     g_list_free_full (self->session_entry_list, g_object_unref);
     G_OBJECT_CLASS (session_list_parent_class)->finalize (object);
 }
@@ -149,6 +152,14 @@ session_list_class_init (SessionListClass *klass)
     object_class->get_property = session_list_get_property;
     object_class->set_property = session_list_set_property;
 
+    obj_properties [PROP_MAX_ABANDONED] =
+        g_param_spec_uint ("max-abandoned",
+                           "max abandoned sessions",
+                           "maximum number of entries permitted in the abandoned state",
+                           0,
+                           SESSION_LIST_MAX_ABANDONED_MAX,
+                           SESSION_LIST_MAX_ABANDONED_DEFAULT,
+                           G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
     obj_properties [PROP_MAX_PER_CONNECTION] =
         g_param_spec_uint ("max-per-connection",
                            "max entries per connection",
@@ -166,29 +177,15 @@ session_list_class_init (SessionListClass *klass)
  * and create the instance.
  */
 SessionList*
-session_list_new (guint  max_per_conn)
+session_list_new (guint  max_per_conn,
+                  guint  max_abandoned)
 {
     g_debug ("session_list_new with max-per-connection: 0x%x", max_per_conn);
     return SESSION_LIST (g_object_new (TYPE_SESSION_LIST,
+                                       "max-abandoned", max_abandoned,
                                        "max-per-connection", max_per_conn,
                                        NULL));
 }
-/*
- * Lock the mutex that protects the hash table object.
- */
-gint
-session_list_lock (SessionList *list)
-{
-    return pthread_mutex_lock (&list->mutex);
-}
-/*
- * Unlock the mutex that protects the list and entry objects.
- */
-gint
-session_list_unlock (SessionList *list)
-{
-    return pthread_mutex_unlock (&list->mutex);
-}
 /*
  * Insert GObject into the session list. We take a reference to the object
  * before we insert the object. When it is removed or if the SessionList
@@ -203,96 +200,17 @@ session_list_insert (SessionList      *list,
     if (list == NULL || entry == NULL) {
         g_error ("session_list_insert passed NULL parameter");
     }
-    session_list_lock (list);
-    if (session_list_is_full_unlocked (list, entry->connection)) {
+    if (session_list_is_full (list, entry->connection)) {
         g_warning ("SessionList: 0x%" PRIxPTR " max_per_connection of %u "
                    "exceeded", (uintptr_t)list, list->max_per_connection);
-        session_list_unlock (list);
         return FALSE;
     }
     g_object_ref (entry);
     list->session_entry_list = g_list_append (list->session_entry_list,
                                                  entry);
-    session_list_unlock (list);
 
     return TRUE;
 }
-/*
- * This function is used with the g_list_find_custom function to find
- * session_entry_t's in the list with a given handle. The first parameter
- * is a reference to an entry in the list. The second is a reference to the
- * handle we're looking for.
- */
-gint
-session_entry_compare (gconstpointer a,
-                       gconstpointer b)
-{
-    if (a == NULL || b == NULL) {
-        g_error ("session_entry_compare_on_handle received NULL parameter");
-    }
-    SessionEntry *entry_a = SESSION_ENTRY (a);
-    SessionEntry *entry_b = SESSION_ENTRY (b);
-
-    if (entry_a < entry_b) {
-        return -1;
-    } else if (entry_a > entry_b) {
-        return 1;
-    } else {
-        return 0;
-    }
-}
-/*
- * This function is used with the g_list_find_custom function to find
- * session_entry_t's in the list with a given handle. The first parameter
- * is a reference to an entry in the list. The second is a reference to the
- * handle we're looking for.
- */
-gint
-session_entry_compare_on_handle (gconstpointer a,
-                                 gconstpointer b)
-{
-    if (a == NULL || b == NULL) {
-        g_error ("session_entry_compare_on_handle received NULL parameter");
-    }
-    SessionEntry *entry_a = SESSION_ENTRY (a);
-    TPM_HANDLE handle_a = session_entry_get_handle (entry_a);
-    TPM_HANDLE handle_b = *(TPM_HANDLE*)b;
-
-    if (handle_a < handle_b) {
-        return -1;
-    } else if (handle_a > handle_b) {
-        return 1;
-    } else {
-        return 0;
-    }
-}
-/*
- * GCompareFunc to search list of session_entry_t structures for a match on
- * the connection object.
- */
-gint
-session_entry_compare_on_connection (gconstpointer a,
-                                     gconstpointer b)
-{
-    gint ret;
-
-    if (a == NULL || b == NULL) {
-        g_error ("session_entry_compare_on_connection received NULL parameter");
-    }
-    SessionEntry    *entry_a          = SESSION_ENTRY (a);
-    Connection      *connection_param = CONNECTION (b);
-    Connection      *connection_entry = session_entry_get_connection (entry_a);
-    if (connection_entry < connection_param) {
-        ret = -1;
-    } else if (connection_entry > connection_param) {
-        ret = 1;
-    } else {
-        ret = 0;
-    }
-    g_object_unref (connection_entry);
-
-    return ret;
-}
 static gboolean
 session_list_remove_custom (SessionList  *list,
                             gconstpointer data,
@@ -301,21 +219,17 @@ session_list_remove_custom (SessionList  *list,
     GList       *list_entry;
     SessionEntry *entry_data;
 
-    session_list_lock (list);
     list_entry = g_list_find_custom (list->session_entry_list, data, func);
     if (list_entry == NULL) {
-        session_list_unlock (list);
         return FALSE;
     }
     entry_data = SESSION_ENTRY (list_entry->data);
     list->session_entry_list = g_list_remove_link (list->session_entry_list,
                                                      list_entry);
     g_object_unref (entry_data);
-    session_list_unlock (list);
 
     return TRUE;
 }
-                          
 /*
  * Remove the entry from the GList. The SessionList assumes that since the
  * entry is in the container it must hold a reference to the object and so
@@ -324,7 +238,7 @@ session_list_remove_custom (SessionList  *list,
  */
 gboolean
 session_list_remove_handle (SessionList      *list,
-                            TPM_HANDLE        handle)
+                            TPM2_HANDLE        handle)
 {
     return session_list_remove_custom (list,
                                        &handle,
@@ -345,8 +259,7 @@ session_list_remove_connection (SessionList      *list,
  * This function is a thin wrapper around the g_list_remove function. Pass it
  * a SessionEntry. It will find it in the list and remove the associated entry
  * and then unref it (to account for the SessionList no longer holding a
- * reference). This function does not lock the SessionList during this
- * operation so the caller will have to do this themselves.
+ * reference).
  */
 void
 session_list_remove (SessionList   *list,
@@ -366,36 +279,31 @@ session_list_remove_last (SessionList *list)
     GList       *list_entry;
     SessionEntry *entry_data;
 
-    session_list_lock (list);
     list_entry = g_list_last (list->session_entry_list);
     if (list_entry == NULL) {
-        session_list_unlock (list);
         return NULL;
     }
     entry_data = SESSION_ENTRY (list_entry->data);
     list->session_entry_list = g_list_remove_link (list->session_entry_list,
                                                      list_entry);
-    session_list_unlock (list);
 
     return entry_data;
 }
 /*
  * This is a lookup function to find an entry in the SessionList given
- * handle. The caller *must* hold the lock on the SessionList. Further they
- * must hold this lock until they are done with the SessionEntry
- * returned. This function increases the reference count on the SessionEntry
- * returned. The caller must decrement the reference count when it is done
- * with the entry.
+ * handle. This function increases the reference count on the
+ * SessionEntry returned. The caller must decrement the reference count
+ * when it is done with the entry.
  */
 SessionEntry*
-session_list_lookup_connection (SessionList   *list,
-                                Connection    *connection)
+session_list_lookup_handle (SessionList   *list,
+                            TPM2_HANDLE     handle)
 {
     GList *list_entry;
 
     list_entry = g_list_find_custom (list->session_entry_list,
-                                      connection,
-                                      session_entry_compare_on_connection);
+                                      &handle,
+                                      session_entry_compare_on_handle);
     if (list_entry != NULL) {
         g_object_ref (list_entry->data);
         return SESSION_ENTRY (list_entry->data);
@@ -404,22 +312,40 @@ session_list_lookup_connection (SessionList   *list,
     }
 }
 /*
- * This is a lookup function to find an entry in the SessionList given
- * handle. The caller *must* hold the lock on the SessionList. Further they
- * must hold this lock until they are done with the SessionListEntry
- * returned. This function increases the reference count on the
- * SessionEntry returned. The caller must decrement the reference count
- * when it is done with the entry.
+ * This type is used as a container for passing a buffer and its size to the
+ * 'session_list_compare_context' callback. This callback must have the
+ * prototype defined as 'GCompareFunc' which forces us to pass only one
+ * arbitrary parameter.
  */
+typedef struct size_buf_ptr {
+    uint8_t *buf;
+    size_t size;
+} size_buf_ptr_t;
+static gint
+session_list_compare_context (gconstpointer a,
+                              gconstpointer b)
+{
+    SessionEntry *entry = SESSION_ENTRY (a);
+    size_buf_ptr_t *size_buf_ptr = (size_buf_ptr_t*)b;
+
+    return session_entry_compare_on_context_client (entry,
+                                                    size_buf_ptr->buf,
+                                                    size_buf_ptr->size);
+}
 SessionEntry*
-session_list_lookup_handle (SessionList   *list,
-                            TPM_HANDLE     handle)
+session_list_lookup_context_client (SessionList *list,
+                                    uint8_t *buf,
+                                    size_t size)
 {
     GList *list_entry;
+    size_buf_ptr_t size_buf_ptr = {
+        .size = size,
+        .buf = buf,
+    };
 
     list_entry = g_list_find_custom (list->session_entry_list,
-                                      &handle,
-                                      session_entry_compare_on_handle);
+                                     &size_buf_ptr,
+                                     session_list_compare_context);
     if (list_entry != NULL) {
         g_object_ref (list_entry->data);
         return SESSION_ENTRY (list_entry->data);
@@ -436,9 +362,7 @@ session_list_size (SessionList *list)
 {
     guint ret;
 
-    session_list_lock (list);
     ret = g_list_length (list->session_entry_list);
-    session_list_unlock (list);
 
     return ret;
 }
@@ -459,11 +383,13 @@ session_list_connection_counter (gpointer data,
                                  gpointer user_data)
 {
     SessionEntry *entry = SESSION_ENTRY (data);
+    Connection *conn = session_entry_get_connection (entry);
     connection_count_data_t *count_data = (connection_count_data_t*)user_data;
 
-    if (count_data->connection == session_entry_get_connection (entry)) {
+    if (count_data->connection == conn) {
         ++count_data->count;
     }
+    g_clear_object (&conn);
 }
 /*
  * Returns the number of entries associated with the provided connection.
@@ -485,8 +411,8 @@ session_list_connection_count (SessionList *list,
  * to max_per_connection.
  */
 gboolean
-session_list_is_full_unlocked (SessionList *session_list,
-                               Connection  *connection)
+session_list_is_full (SessionList *session_list,
+                      Connection  *connection)
 {
     size_t session_count;
     gboolean ret;
@@ -502,17 +428,6 @@ session_list_is_full_unlocked (SessionList *session_list,
     }
     return ret;
 }
-gboolean
-session_list_is_full (SessionList *session_list,
-                      Connection  *connection)
-{
-    gboolean ret;
-
-    session_list_lock (session_list);
-    ret = session_list_is_full_unlocked (session_list, connection);
-    session_list_unlock (session_list);
-    return ret;
-}
 /*
  */
 static void
@@ -520,6 +435,7 @@ session_list_dump_entry (gpointer data,
                          gpointer user_data)
 {
     SessionEntry *entry = SESSION_ENTRY (data);
+    UNUSED_PARAM(user_data);
 
     session_entry_prettyprint (entry);
 }
@@ -529,11 +445,9 @@ void
 session_list_prettyprint (SessionList *list)
 {
     g_debug ("SessionList: 0x%" PRIxPTR, (uintptr_t)list);
-    session_list_lock (list);
     g_list_foreach (list->session_entry_list,
                      session_list_dump_entry,
                      NULL);
-    session_list_unlock (list);
 }
 /*
  */
@@ -546,3 +460,103 @@ session_list_foreach (SessionList *list,
                      func,
                      user_data);
 }
+/*
+ * Find the associated SessionEntry in the list.
+ * Check that the SessionEntry has the same
+ */
+gboolean
+session_list_abandon_handle (SessionList *list,
+                             Connection *connection,
+                             TPM2_HANDLE handle)
+{
+    SessionEntry *entry = NULL;
+
+    entry = session_list_lookup_handle (list, handle);
+    if (entry == NULL) {
+        g_debug ("%s: Handle 0x%08" PRIx32 " doesn't exist in SessionList 0x%"
+                 PRIxPTR, __func__, handle, (uintptr_t)list);
+        return FALSE;
+    }
+    if (session_entry_compare_on_connection (entry, connection)) {
+        g_warning ("%s: Connection 0x%" PRIxPTR " attempted to abandon "
+                   "SessionEntry 0x%" PRIxPTR "with handle 0x%08" PRIx32,
+                   __func__, (uintptr_t)connection, (uintptr_t)entry, handle);
+        g_clear_object (&entry);
+        return FALSE;
+    }
+    session_entry_abandon (entry);
+    g_queue_push_head (list->abandoned_queue, entry);
+    g_clear_object (&entry);
+
+    return TRUE;
+}
+/*
+ * SessionEntry objects can be claimed when they're in two states:
+ * - If the SessionEntry has been abandoned by another connection it will be
+ *   in the 'abandoned_queue'. In this case we remove it from the
+ *   'abandoned_queue', set the state to 'LOADED' and associate the provided
+ *   connection with the object.
+ * - If the SessionEntry has been saved BY THE CLIENT then it will *not* be
+ *   in the 'abandoned_queue'. In this case we find the SessionEntry in the
+ *   'session_entry_list' and change the connection.
+ */
+gboolean
+session_list_claim (SessionList *list,
+                    SessionEntry *entry,
+                    Connection  *connection)
+{
+    GList *link = NULL;
+
+    link = g_queue_find (list->abandoned_queue, entry);
+    if (link != NULL) {
+        g_assert (link->data == entry);
+        g_debug ("%s: GQueue of abandoned sessions does not contain "
+                 "SessionEntry: 0x%08" PRIxPTR, __func__, (uintptr_t)entry);
+        session_entry_set_state (entry, SESSION_ENTRY_LOADED);
+        session_entry_set_connection (entry, connection);
+        g_queue_remove (list->abandoned_queue, link->data);
+        return TRUE;
+    }
+    link = g_list_find (list->session_entry_list, entry);
+    if (link != NULL) {
+        g_assert (link->data == entry);
+        g_debug ("%s: SessionEntry object 0x%" PRIxPTR " found in SessionList",
+                 __func__, (uintptr_t)entry);
+        session_entry_set_state (entry, SESSION_ENTRY_LOADED);
+        session_entry_set_connection (entry, connection);
+    } else {
+        return FALSE;
+    }
+    return TRUE;
+}
+/*
+ * Remove oldest from abandoned queue and call the caller provided function
+ * on it.
+ */
+gboolean
+session_list_prune_abandoned (SessionList *list,
+                              PruneFunc func,
+                              gpointer data)
+{
+    SessionEntry *entry = NULL;
+    gboolean ret = FALSE;
+
+    g_debug ("%s: SessionList 0x%" PRIxPTR ", PruneFunc 0x%" PRIxPTR ", "
+             "data 0x%" PRIxPTR, __func__, (uintptr_t)list, (uintptr_t)func,
+             (uintptr_t)data);
+    if (g_queue_get_length (list->abandoned_queue) <= list->max_abandoned) {
+        g_debug ("%s: abandoned_queue has not exceeded 'max_abandoned', "
+                 "nothing to do.", __func__);
+        return TRUE;
+    }
+    entry = g_queue_pop_tail (list->abandoned_queue);
+    if (entry == NULL) {
+        g_debug ("%s: Abandoned queue is empty.", __func__);
+        return TRUE;
+    }
+    g_object_ref (entry);
+    session_list_remove (list, entry);
+    ret = func (entry, data);
+    g_clear_object (&entry);
+    return ret;
+}
