@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, Intel Corporation
+ * Copyright (c) 2017 - 2018, Intel Corporation
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,10 +33,10 @@
 #include <poll.h>
 #include <string.h>
 
-#include <sapi/tpm20.h>
+#include <tss2/tss2_tpm2_types.h>
 
 #include "tabrmd.h"
-#include "tcti-tabrmd.h"
+#include "tss2-tcti-tabrmd.h"
 #include "tcti-tabrmd-priv.h"
 #include "tpm2-header.h"
 #include "util.h"
@@ -44,7 +44,7 @@
 static TSS2_RC
 tss2_tcti_tabrmd_transmit (TSS2_TCTI_CONTEXT *context,
                            size_t             size,
-                           uint8_t           *command)
+                           const uint8_t      *command)
 {
     ssize_t write_ret;
     TSS2_RC tss2_ret = TSS2_RC_SUCCESS;
@@ -81,7 +81,7 @@ tss2_tcti_tabrmd_transmit (TSS2_TCTI_CONTEXT *context,
         tss2_ret = TSS2_TCTI_RC_NO_CONNECTION;
         break;
     default:
-        if (write_ret == size) {
+        if (write_ret == (ssize_t) size) {
             TSS2_TCTI_TABRMD_STATE (context) = TABRMD_STATE_RECEIVE;
         } else {
             g_debug ("tss2_tcti_tabrmd_transmit: short write");
@@ -165,7 +165,7 @@ tcti_tabrmd_poll (int        fd,
     int ret;
     int errno_tmp;
 
-    ret = TEMP_FAILURE_RETRY (poll (pollfds,
+    ret = TABRMD_ERRNO_EINTR_RETRY (poll (pollfds,
                                     sizeof (pollfds) / sizeof (struct pollfd),
                                     timeout));
     errno_tmp = errno;
@@ -175,7 +175,7 @@ tcti_tabrmd_poll (int        fd,
                  errno_tmp, strerror (errno_tmp));
         return errno_tmp;
     case 0:
-        g_debug ("poll timed out after %" PRId32 " miniseconds", timeout);
+        g_debug ("poll timed out after %" PRId32 " milliseconds", timeout);
         return -1;
     default:
         g_debug ("poll has %d fds ready", ret);
@@ -390,6 +390,8 @@ tss2_tcti_tabrmd_set_locality (TSS2_TCTI_CONTEXT *context,
 void
 init_tcti_data (TSS2_TCTI_CONTEXT *context)
 {
+    memset (context, 0, sizeof (TSS2_TCTI_TABRMD_CONTEXT));
+
     TSS2_TCTI_MAGIC (context)            = TSS2_TCTI_TABRMD_MAGIC;
     TSS2_TCTI_VERSION (context)          = TSS2_TCTI_TABRMD_VERSION;
     TSS2_TCTI_TABRMD_STATE (context)     = TABRMD_STATE_TRANSMIT;
@@ -428,55 +430,87 @@ _out:
     return _ret != NULL;
 }
 
-TSS2_RC
-tss2_tcti_tabrmd_init_full (TSS2_TCTI_CONTEXT      *context,
-                            size_t                 *size,
-                            TCTI_TABRMD_DBUS_TYPE   bus_type,
-                            const char             *bus_name)
-{
-    GBusType g_bus_type;
-    GError *error = NULL;
-    GSocket *sock;
-    GVariant *fds_variant;
-    guint64 id;
-    GUnixFDList *fd_list;
-    gboolean call_ret;
+typedef struct {
+    char *name;
+    GBusType type;
+} bus_name_type_entry_t;
 
-    if (context == NULL && size == NULL) {
-        return TSS2_TCTI_RC_BAD_VALUE;
+static const bus_name_type_entry_t bus_name_type_map[] = {
+    {
+        .name = "session",
+        .type = G_BUS_TYPE_SESSION,
+    },
+    {
+        .name = "system",
+        .type = G_BUS_TYPE_SYSTEM,
+    },
+};
+#define BUS_NAME_TYPE_MAP_LENGTH (sizeof (bus_name_type_map) / sizeof (bus_name_type_entry_t))
+GBusType
+tabrmd_bus_type_from_str (const char* const bus_type)
+{
+    size_t i;
+    g_debug ("BUS_NAME_TYPE_MAP_LENGTH: %zu", BUS_NAME_TYPE_MAP_LENGTH);
+    g_debug ("looking up type for bus_type string: %s", bus_type);
+    for (i = 0; i < BUS_NAME_TYPE_MAP_LENGTH; ++i) {
+        if (strcmp (bus_name_type_map [i].name, bus_type) == 0) {
+            g_debug ("matched bus_type string \"%s\" to type %d",
+                     bus_name_type_map [i].name,
+                     bus_name_type_map [i].type);
+            return bus_name_type_map [i].type;
+        }
     }
-    if (context == NULL && size != NULL) {
-        *size = sizeof (TSS2_TCTI_TABRMD_CONTEXT);
+    g_debug ("no match for bus_type string %s", bus_type);
+    return G_BUS_TYPE_NONE;
+}
+
+TSS2_RC
+tabrmd_kv_callback (const key_value_t *key_value,
+                    gpointer user_data)
+{
+    tabrmd_conf_t *tabrmd_conf = (tabrmd_conf_t*)user_data;
+
+    g_debug ("%s with key_value: 0x%" PRIxPTR " and user_data: 0x%" PRIxPTR,
+             __func__, (uintptr_t)key_value, (uintptr_t)user_data);
+    if (key_value == NULL || user_data == NULL) {
+        g_warning ("%s passed NULL parameter", __func__);
+        return TSS2_TCTI_RC_GENERAL_FAILURE;
+    }
+    g_debug ("key: %s / value: %s\n", key_value->key, key_value->value);
+    if (strcmp (key_value->key, "bus_name") == 0) {
+        tabrmd_conf->bus_name = key_value->value;
         return TSS2_RC_SUCCESS;
-    }
-    if (bus_name == NULL) {
+    } else if (strcmp (key_value->key, "bus_type") == 0) {
+        tabrmd_conf->bus_type = tabrmd_bus_type_from_str (key_value->value);
+        if (tabrmd_conf->bus_type == G_BUS_TYPE_NONE) {
+            return TSS2_TCTI_RC_BAD_VALUE;
+        }
+        return TSS2_RC_SUCCESS;
+    } else {
         return TSS2_TCTI_RC_BAD_VALUE;
     }
-    switch (bus_type) {
-    case TCTI_TABRMD_DBUS_TYPE_SESSION:
-        g_bus_type = G_BUS_TYPE_SESSION;
-        break;
-    case TCTI_TABRMD_DBUS_TYPE_SYSTEM:
-        g_bus_type = G_BUS_TYPE_SYSTEM;
-        break;
-    default:
-        return TSS2_TCTI_RC_BAD_VALUE;
-    }
-    /* Register dbus error mapping for tabrmd. Gets us RCs from Gerror codes */
-    TABRMD_ERROR;
-    init_tcti_data (context);
-    TSS2_TCTI_TABRMD_PROXY (context) =
-        tcti_tabrmd_proxy_new_for_bus_sync (
-            g_bus_type,
-            G_DBUS_PROXY_FLAGS_NONE,
-            bus_name,
-            TABRMD_DBUS_PATH, /* object */
-            NULL,                          /* GCancellable* */
-            &error);
-    if (TSS2_TCTI_TABRMD_PROXY (context) == NULL) {
-        g_critical ("failed to allocate dbus proxy object: %s", error->message);
-        return TSS2_TCTI_RC_NO_CONNECTION;
-    }
+}
+
+/*
+ * Establish a connection with the daemon. This includes calling the
+ * CreateConnection dbus method, extracting the file descriptor used for
+ * sending commands and receiving responses, and extracting the connection
+ * ID used when sending commands over the dbus interface.
+ *
+ * The proxy object in the context structure must be created / valid before
+ * calling this function.
+ */
+TSS2_RC
+tcti_tabrmd_connect (TSS2_TCTI_CONTEXT *context)
+{
+    GError *error = NULL;
+    GSocket *sock = NULL;
+    GUnixFDList *fd_list = NULL;
+    GVariant *fds_variant = NULL;
+    gboolean call_ret;
+    guint64 id;
+    TSS2_RC rc = TSS2_RC_SUCCESS;
+
     call_ret = tcti_tabrmd_call_create_connection_sync_fdlist (
         TSS2_TCTI_TABRMD_PROXY (context),
         &fds_variant,
@@ -487,39 +521,123 @@ tss2_tcti_tabrmd_init_full (TSS2_TCTI_CONTEXT      *context,
     if (call_ret == FALSE) {
         g_warning ("Failed to create connection with service: %s",
                  error->message);
-        return TSS2_TCTI_RC_NO_CONNECTION;
+        rc = TSS2_TCTI_RC_NO_CONNECTION;
+        goto out;
     }
     if (fd_list == NULL) {
-        g_error ("call to CreateConnection returned a NULL GUnixFDList");
+        g_critical ("call to CreateConnection returned a NULL GUnixFDList");
+        rc = TSS2_TCTI_RC_NO_CONNECTION;
+        goto out;
     }
     gint num_handles = g_unix_fd_list_get_length (fd_list);
     if (num_handles != 1) {
-        g_error ("CreateConnection expected to return 1 handles, received %d",
-                 num_handles);
+        g_critical ("CreateConnection expected to return 1 handles, received %d",
+                    num_handles);
+        rc = TSS2_TCTI_RC_GENERAL_FAILURE;
+        goto out;
     }
     gint fd = g_unix_fd_list_get (fd_list, 0, &error);
     if (fd == -1) {
-        g_error ("unable to get receive handle from GUnixFDList: %s",
-                 error->message);
+        g_critical ("unable to get receive handle from GUnixFDList: %s",
+                    error->message);
+        rc = TSS2_TCTI_RC_GENERAL_FAILURE;
+        goto out;
     }
     sock = g_socket_new_from_fd (fd, NULL);
     TSS2_TCTI_TABRMD_SOCK_CONNECT (context) = \
         g_socket_connection_factory_create_connection (sock);
-    g_object_unref (sock);
     TSS2_TCTI_TABRMD_ID (context) = id;
-    g_debug ("initialized tabrmd TCTI context with id: 0x%" PRIx64,
-             TSS2_TCTI_TABRMD_ID (context));
-    g_object_unref (fd_list);
-
-    return TSS2_RC_SUCCESS;
+out:
+    g_clear_pointer (&fds_variant, g_variant_unref);
+    g_clear_error (&error);
+    g_clear_object (&sock);
+    g_clear_object (&fd_list);
+    return rc;
 }
 
+/*
+ * The longest configuration string we'll take. Each dbus name can be 255
+ * characters long (see dbus spec). The bus_types that we support are
+ * 'system' or 'session' (255 + 7 = 262). 'bus_type=' and 'bus_name=' are
+ * each another 9 characters for a total of 280.
+ */
+#define CONF_STRING_MAX 280
 TSS2_RC
-tss2_tcti_tabrmd_init (TSS2_TCTI_CONTEXT *context,
-                       size_t            *size)
+Tss2_Tcti_Tabrmd_Init (TSS2_TCTI_CONTEXT *context,
+                       size_t            *size,
+                       const char        *conf)
 {
-    return tss2_tcti_tabrmd_init_full (context,
-                                       size,
-                                       TCTI_TABRMD_DBUS_TYPE_DEFAULT,
-                                       TCTI_TABRMD_DBUS_NAME_DEFAULT);
+    GError *error = NULL;
+    size_t conf_len;
+    char *conf_copy = NULL;
+    TSS2_RC rc;
+    tabrmd_conf_t tabrmd_conf = TABRMD_CONF_INIT_DEFAULT;
+
+    if (context == NULL && size != NULL) {
+        *size = sizeof (TSS2_TCTI_TABRMD_CONTEXT);
+        return TSS2_RC_SUCCESS;
+    }
+    if (size == NULL) {
+        return TSS2_TCTI_RC_BAD_VALUE;
+    }
+    if (conf != NULL) {
+        conf_len = strlen (conf);
+        if (conf_len > CONF_STRING_MAX) {
+            return TSS2_TCTI_RC_BAD_VALUE;
+        }
+        conf_copy = g_strdup (conf);
+        if (conf_copy == NULL) {
+            g_critical ("Failed to duplicate config string: %s", strerror (errno));
+            return TSS2_TCTI_RC_GENERAL_FAILURE;
+        }
+        rc = parse_key_value_string (conf_copy,
+                                     tabrmd_kv_callback,
+                                     &tabrmd_conf);
+        if (rc != TSS2_RC_SUCCESS) {
+            goto out;
+        }
+    }
+    /* Register dbus error mapping for tabrmd. Gets us RCs from Gerror codes */
+    TABRMD_ERROR;
+    init_tcti_data (context);
+    TSS2_TCTI_TABRMD_PROXY (context) =
+        tcti_tabrmd_proxy_new_for_bus_sync (tabrmd_conf.bus_type,
+                                            G_DBUS_PROXY_FLAGS_NONE,
+                                            tabrmd_conf.bus_name,
+                                            TABRMD_DBUS_PATH,
+                                            NULL,
+                                            &error);
+    if (TSS2_TCTI_TABRMD_PROXY (context) == NULL) {
+        g_critical ("failed to allocate dbus proxy object: %s", error->message);
+        rc = TSS2_TCTI_RC_NO_CONNECTION;
+        goto out;
+    }
+    rc = tcti_tabrmd_connect (context);
+    if (rc == TSS2_RC_SUCCESS) {
+        g_debug ("initialized tabrmd TCTI context with id: 0x%" PRIx64,
+                 TSS2_TCTI_TABRMD_ID (context));
+    }
+out:
+    g_clear_pointer (&conf_copy, g_free);
+    g_clear_error (&error);
+
+    return rc;
+}
+
+/* public info structure */
+static const TSS2_TCTI_INFO tss2_tcti_info = {
+    .version = TSS2_TCTI_TABRMD_VERSION,
+    .name = "tcti-abrmd",
+    .description = "TCTI module for communication with tabrmd.",
+    .config_help = "This conf string is a series of key / value pairs " \
+        "where keys and values are separated by the '=' character and " \
+        "each pair is separated by the ',' character. Valid keys are " \
+        "\"bus_name\" and \"bus_type\".",
+    .init = Tss2_Tcti_Tabrmd_Init,
+};
+
+const TSS2_TCTI_INFO*
+Tss2_Tcti_Info (void)
+{
+    return &tss2_tcti_info;
 }
