@@ -26,6 +26,10 @@
  */
 #include <inttypes.h>
 #include <stdint.h>
+#include <string.h>
+
+#include <tss2/tss2_tpm2_types.h>
+#include <tss2/tss2_mu.h>
 
 #include "tpm2-command.h"
 #include "tpm2-header.h"
@@ -33,17 +37,17 @@
 /* macros to make getting at fields in the command more simple */
 
 #define HANDLE_AREA_OFFSET TPM_HEADER_SIZE
-#define HANDLE_OFFSET(i) (HANDLE_AREA_OFFSET + sizeof (TPM_HANDLE) * i)
-#define HANDLE_END_OFFSET(i) (HANDLE_OFFSET (i) + sizeof (TPM_HANDLE))
+#define HANDLE_OFFSET(i) (HANDLE_AREA_OFFSET + sizeof (TPM2_HANDLE) * i)
+#define HANDLE_END_OFFSET(i) (HANDLE_OFFSET (i) + sizeof (TPM2_HANDLE))
 #define HANDLE_GET(buffer, count) \
-    (*(TPM_HANDLE*)(&buffer [HANDLE_OFFSET (count)]))
+    (*(TPM2_HANDLE*)(&buffer [HANDLE_OFFSET (count)]))
 /*
  * Offset of capability field in TPM2_GetCapability command buffer is
  * immediately after the header.
  */
 #define CAP_OFFSET TPM_HEADER_SIZE
-#define CAP_END_OFFSET (CAP_OFFSET + sizeof (TPM_CAP))
-#define CAP_GET(buffer) (*(TPM_CAP*)(&buffer [CAP_OFFSET]))
+#define CAP_END_OFFSET (CAP_OFFSET + sizeof (TPM2_CAP))
+#define CAP_GET(buffer) (*(TPM2_CAP*)(&buffer [CAP_OFFSET]))
 /*
  * Offset of property field in TPM2_GetCapability command buffer is
  * immediately after the capability field.
@@ -63,7 +67,7 @@
  * authorization area.
  */
 #define AUTH_AREA_OFFSET(cmd) \
-    (TPM_HEADER_SIZE + (tpm2_command_get_handle_count (cmd) * sizeof (TPM_HANDLE)))
+    (TPM_HEADER_SIZE + (tpm2_command_get_handle_count (cmd) * sizeof (TPM2_HANDLE)))
 #define AUTH_AREA_SIZE_OFFSET(cmd) AUTH_AREA_OFFSET (cmd)
 #define AUTH_AREA_SIZE_END_OFFSET(cmd) \
     (AUTH_AREA_SIZE_OFFSET (cmd) + sizeof (UINT32))
@@ -81,9 +85,9 @@
  */
 #define AUTH_HANDLE_OFFSET(index) (index + 0)
 #define AUTH_HANDLE_END_OFFSET(index) \
-    (AUTH_HANDLE_OFFSET(index) + sizeof (TPM_HANDLE))
+    (AUTH_HANDLE_OFFSET(index) + sizeof (TPM2_HANDLE))
 #define AUTH_GET_HANDLE(cmd, index) \
-    (be32toh (*(TPM_HANDLE*)&cmd->buffer [AUTH_HANDLE_OFFSET (index)]))
+    (be32toh (*(TPM2_HANDLE*)&cmd->buffer [AUTH_HANDLE_OFFSET (index)]))
 /* nonce */
 #define AUTH_NONCE_SIZE_OFFSET(index) (AUTH_HANDLE_END_OFFSET (index))
 #define AUTH_NONCE_SIZE_END_OFFSET(index) \
@@ -104,7 +108,7 @@
 #define AUTH_SESSION_ATTRS_END_OFFSET(cmd, index) \
     (AUTH_SESSION_ATTRS_OFFSET (cmd, index) + sizeof (UINT8))
 #define AUTH_GET_SESSION_ATTRS(cmd, index) \
-    ((TPMA_SESSION)(UINT32)cmd->buffer [AUTH_SESSION_ATTRS_OFFSET (cmd, index)])
+    ((TPMA_SESSION)(UINT8)cmd->buffer [AUTH_SESSION_ATTRS_OFFSET (cmd, index)])
 /* authorization */
 #define AUTH_AUTH_SIZE_OFFSET(cmd, index) \
     (AUTH_SESSION_ATTRS_END_OFFSET (cmd, index))
@@ -159,7 +163,9 @@ tpm2_command_set_property (GObject        *object,
             break;
         }
         self->connection = g_value_get_object (value);
-        g_object_ref (self->connection);
+        if (self->connection) {
+            g_object_ref (self->connection);
+        }
         break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -180,7 +186,7 @@ tpm2_command_get_property (GObject     *object,
     g_debug ("tpm2_command_get_property: 0x%" PRIxPTR, (uintptr_t)self);
     switch (property_id) {
     case PROP_ATTRIBUTES:
-        g_value_set_uint (value, self->attributes.val);
+        g_value_set_uint (value, self->attributes);
         break;
     case PROP_BUFFER:
         g_value_set_pointer (value, self->buffer);
@@ -220,7 +226,10 @@ tpm2_command_finalize (GObject *obj)
 }
 static void
 tpm2_command_init (Tpm2Command *command)
-{ /* noop */ }
+{
+    UNUSED_PARAM(command);
+    /* noop */
+}
 /**
  * Boilerplate GObject initialization. Get a pointer to the parent class,
  * setup a finalize function.
@@ -284,6 +293,58 @@ tpm2_command_new (Connection     *connection,
                                        "connection", connection,
                                        NULL));
 }
+#define CONTEXT_SAVE_CMD_SIZE (TPM_HEADER_SIZE + sizeof (TPM2_HANDLE))
+Tpm2Command*
+tpm2_command_new_context_save (TPM2_HANDLE handle)
+{
+    TSS2_RC rc;
+    uint8_t *buf = g_malloc0 (CONTEXT_SAVE_CMD_SIZE);
+    size_t offset = TPM_HEADER_SIZE;
+
+    rc = tpm2_header_init (buf,
+                           CONTEXT_SAVE_CMD_SIZE,
+                           TPM2_ST_NO_SESSIONS,
+                           CONTEXT_SAVE_CMD_SIZE,
+                           TPM2_CC_ContextSave);
+    if (rc != TSS2_RC_SUCCESS) {
+        goto err_out;
+    }
+    rc = Tss2_MU_TPM2_HANDLE_Marshal (handle,
+                                      buf,
+                                      CONTEXT_SAVE_CMD_SIZE,
+                                      &offset);
+    if (rc != TSS2_RC_SUCCESS) {
+        goto err_out;
+    }
+    /* TPMA_CC here is hard coded to the appropriate value for ContextSave */
+    return tpm2_command_new (NULL, buf, CONTEXT_SAVE_CMD_SIZE, 0x02000162);
+
+err_out:
+    g_warning ("%s: failed", __func__);
+    g_free (buf);
+    return NULL;
+}
+Tpm2Command*
+tpm2_command_new_context_load (uint8_t *buf,
+                               size_t size)
+{
+    TSS2_RC rc;
+    UINT32 size_new = TPM_HEADER_SIZE + size;
+    uint8_t *buf_tmp = g_malloc0 (size_new);
+
+    rc = tpm2_header_init (buf_tmp,
+                           size_new,
+                           TPM2_ST_NO_SESSIONS,
+                           size_new,
+                           TPM2_CC_ContextLoad);
+    if (rc != TSS2_RC_SUCCESS) {
+        g_free (buf_tmp);
+        return NULL;
+    }
+    memcpy (&buf_tmp [TPM_HEADER_SIZE], buf, size);
+    /* TPMA_CC here is hard coded to the appropriate value for ContextLoad */
+    return tpm2_command_new (NULL, buf_tmp, size_new, 0x10000161);
+}
 /* Simple "getter" to expose the attributes associated with the command. */
 TPMA_CC
 tpm2_command_get_attributes (Tpm2Command *command)
@@ -299,7 +360,7 @@ tpm2_command_get_buffer (Tpm2Command *command)
 }
 /**
  */
-TPM_CC
+TPM2_CC
 tpm2_command_get_code (Tpm2Command *command)
 {
     return get_command_code (command->buffer);
@@ -328,7 +389,9 @@ tpm2_command_get_tag (Tpm2Command *command)
 Connection*
 tpm2_command_get_connection (Tpm2Command *command)
 {
-    g_object_ref (command->connection);
+    if (command->connection) {
+        g_object_ref (command->connection);
+    }
     return command->connection;
 }
 /* Return the number of handles in the command. */
@@ -342,8 +405,8 @@ tpm2_command_get_handle_count (Tpm2Command *command)
         g_warning ("tpm2_command_get_handle_count received NULL parameter");
         return 0;
     }
-    tmp = tpm2_command_get_attributes (command).val;
-    return (guint8)((tmp & TPMA_CC_CHANDLES) >> 25);
+    tmp = tpm2_command_get_attributes (command);
+    return (guint8)((tmp & TPMA_CC_CHANDLES_MASK) >> TPMA_CC_CHANDLES_SHIFT);
 }
 /*
  * Simple function to access handles in the provided Tpm2Command. The
@@ -351,7 +414,7 @@ tpm2_command_get_handle_count (Tpm2Command *command)
  * which is effectively an array. If the handle_number requests a handle
  * beyond the end of this array 0 is returned.
  */
-TPM_HANDLE
+TPM2_HANDLE
 tpm2_command_get_handle (Tpm2Command *command,
                          guint8       handle_number)
 {
@@ -375,7 +438,7 @@ tpm2_command_get_handle (Tpm2Command *command,
  */
 gboolean
 tpm2_command_set_handle (Tpm2Command *command,
-                         TPM_HANDLE   handle,
+                         TPM2_HANDLE   handle,
                          guint8       handle_number)
 {
     guint8 real_count;
@@ -400,7 +463,7 @@ tpm2_command_set_handle (Tpm2Command *command,
  */
 gboolean
 tpm2_command_get_handles (Tpm2Command *command,
-                          TPM_HANDLE   handles[],
+                          TPM2_HANDLE   handles[],
                           size_t      *count)
 {
     guint8 real_count;
@@ -420,7 +483,7 @@ tpm2_command_get_handles (Tpm2Command *command,
         handles[i] = tpm2_command_get_handle (command, i);
 
         if (handles[i] == 0) {
-            // no more handle could be extracted
+            /* no more handles could be extracted */
             break;
         }
     }
@@ -437,7 +500,7 @@ tpm2_command_get_handles (Tpm2Command *command,
  */
 gboolean
 tpm2_command_set_handles (Tpm2Command *command,
-                          TPM_HANDLE   handles[],
+                          TPM2_HANDLE   handles[],
                           guint8       count)
 {
     guint8 real_count, i;
@@ -472,22 +535,22 @@ tpm2_command_set_handles (Tpm2Command *command,
  * provide a special function for getting at this single handle.
  * Use this function with caution.
  */
-TPM_RC
+TSS2_RC
 tpm2_command_get_flush_handle (Tpm2Command *command,
-                               TPM_HANDLE  *handle)
+                               TPM2_HANDLE  *handle)
 {
     if (command == NULL || handle == NULL) {
         g_error ("tpm2_command_get_flush_handle passed null parameter");
     }
-    if (tpm2_command_get_code (command) != TPM_CC_FlushContext) {
+    if (tpm2_command_get_code (command) != TPM2_CC_FlushContext) {
         g_warning ("tpm2_command_get_flush_handle called with wrong command");
         *handle = 0;
-        return RM_RC (TPM_RC_TYPE);
+        return RM_RC (TPM2_RC_TYPE);
     }
-    if (command->buffer_size < TPM_HEADER_SIZE + sizeof (TPM_HANDLE)) {
+    if (command->buffer_size < TPM_HEADER_SIZE + sizeof (TPM2_HANDLE)) {
         g_warning ("%s: command buffer_size insufficient", __func__);
         *handle = 0;
-        return RM_RC (TPM_RC_INSUFFICIENT);
+        return RM_RC (TPM2_RC_INSUFFICIENT);
     }
     /*
      * Despite not technically being in the "handle area" of the command we
@@ -503,14 +566,14 @@ tpm2_command_get_flush_handle (Tpm2Command *command,
  * GetCapability command this function will extract the 'capability' field.
  * On error 0 is returned.
  */
-TPM_CAP
+TPM2_CAP
 tpm2_command_get_cap (Tpm2Command *command)
 {
     if (command == NULL) {
         g_warning ("tpm2_command_get_cap passed NULL parameter");
         return 0;
     }
-    if (tpm2_command_get_code (command) != TPM_CC_GetCapability) {
+    if (tpm2_command_get_code (command) != TPM2_CC_GetCapability) {
         g_warning ("tpm2_command_get_cap provided a Tpm2Command buffer "
                    "containing the wrong command code.");
         return 0;
@@ -519,7 +582,7 @@ tpm2_command_get_cap (Tpm2Command *command)
         g_warning ("%s insufficient buffer", __func__);
         return 0;
     }
-    return (TPM_CAP)be32toh (CAP_GET (tpm2_command_get_buffer (command)));
+    return (TPM2_CAP)be32toh (CAP_GET (tpm2_command_get_buffer (command)));
 }
 /*
  * When provided with a Tpm2Command that represents a call to the
@@ -533,7 +596,7 @@ tpm2_command_get_prop (Tpm2Command *command)
         g_warning ("tpm2_command_get_prop passed NULL parameter");
         return 0;
     }
-    if (tpm2_command_get_code (command) != TPM_CC_GetCapability) {
+    if (tpm2_command_get_code (command) != TPM2_CC_GetCapability) {
         g_warning ("tpm2_command_get_cap provided a Tpm2Command buffer "
                    "containing the wrong command code.");
         return 0;
@@ -553,10 +616,10 @@ UINT32
 tpm2_command_get_prop_count (Tpm2Command *command)
 {
     if (command == NULL) {
-        g_warning ("tpm2_command_get_prop_count assed NULL parameter");
+        g_warning ("tpm2_command_get_prop_count passed NULL parameter");
         return 0;
     }
-    if (tpm2_command_get_code (command) != TPM_CC_GetCapability) {
+    if (tpm2_command_get_code (command) != TPM2_CC_GetCapability) {
         g_warning ("tpm2_command_get_cap provided a Tpm2Command buffer "
                    "containing the wrong command code.");
         return 0;
@@ -568,8 +631,8 @@ tpm2_command_get_prop_count (Tpm2Command *command)
     return (UINT32)be32toh (PROPERTY_COUNT_GET (tpm2_command_get_buffer (command)));
 }
 /*
- * This is a convencience function to keep from having to compare the tag
- * value to TPM_ST_(NO_)?_SESSIONS repeatedly.
+ * This is a convenience function to keep from having to compare the tag
+ * value to TPM2_ST_(NO_)?_SESSIONS repeatedly.
  */
 gboolean
 tpm2_command_has_auths (Tpm2Command *command)
@@ -578,7 +641,7 @@ tpm2_command_has_auths (Tpm2Command *command)
         g_warning ("tpm2_command_has_auths passed NULL parameter");
         return FALSE;
     }
-    if (tpm2_command_get_tag (command) == TPM_ST_NO_SESSIONS) {
+    if (tpm2_command_get_tag (command) == TPM2_ST_NO_SESSIONS) {
         return FALSE;
     } else {
         return TRUE;
@@ -618,7 +681,7 @@ tpm2_command_get_auths_size (Tpm2Command *command)
  * auth area that begins at offset 'auth_offset'. Any failure to read this
  * value will return 0.
  */
-TPM_HANDLE
+TPM2_HANDLE
 tpm2_command_get_auth_handle (Tpm2Command *command,
                               size_t       auth_index)
 {
@@ -639,13 +702,13 @@ tpm2_command_get_auth_attrs (Tpm2Command *command,
     size_t attrs_end;
 
     if (command == NULL) {
-        return (TPMA_SESSION)(UINT32)0;
+        return (TPMA_SESSION)(UINT8)0;
     }
     attrs_end = AUTH_SESSION_ATTRS_END_OFFSET (command, auth_offset);
     if (attrs_end > command->buffer_size) {
         g_warning ("%s attempt to access session attributes overruns command "
                    "buffer", __func__);
-        return (TPMA_SESSION)(UINT32)0;
+        return (TPMA_SESSION)(UINT8)0;
     }
     return AUTH_GET_SESSION_ATTRS (command, auth_offset);
 }
